@@ -33,12 +33,14 @@
 
 
 static gboolean updating = FALSE;
+static gint prev_value;
 
 
 static void
 cb_ascii_select_clicked (GtkDialog *dialog, gint id, gpointer user_data)
 {
     gchar *text;
+    gint current_pos;
     
     if (id != 100) {
         gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -46,20 +48,9 @@ cb_ascii_select_clicked (GtkDialog *dialog, gint id, gpointer user_data)
     }
 
     text = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
-    if (mainapp->insert_at_end == FALSE)
-    {
-	int current_pos;
-	
-	current_pos = gtk_editable_get_position
-		    (GTK_EDITABLE(mainapp->entry));
-	gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text,
-              strlen (text), &current_pos);
-    } else {
-        gint current_pos;
-        gtk_editable_set_position (GTK_EDITABLE (mainapp->entry), -1);
-        current_pos = gtk_editable_get_position (GTK_EDITABLE (mainapp->entry));
-        gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text, strlen(text), &current_pos);
-    }
+    gtk_editable_set_position (GTK_EDITABLE (mainapp->entry), -1);
+    current_pos = gtk_editable_get_position (GTK_EDITABLE (mainapp->entry));
+    gtk_editable_insert_text (GTK_EDITABLE (mainapp->entry), text, strlen(text), &current_pos);
        
 }
 
@@ -98,8 +89,13 @@ cb_ascii_select_spin_changed (GtkEditable *edit, gpointer user_data)
     	return;
     i = atoi (text);
     g_free (text);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (edit), i);
+    if(i > 255)
+    {	     
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON (edit), prev_value);
+        i = prev_value;
+    }	
 
+    prev_value = i;
     n=g_unichar_to_utf8(i, buf);
     buf[n]=0;
 
@@ -122,7 +118,7 @@ ascii_select_init (AsciiSelect *obj)
 					       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					       _("_Insert"), 100,
 					       NULL);
-    
+    gtk_dialog_set_default_response (GTK_DIALOG (obj->window), GTK_RESPONSE_CLOSE);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj->window)->vbox),
       gtk_label_new (_("Character code:")), FALSE, FALSE, 0);
 
@@ -130,6 +126,8 @@ ascii_select_init (AsciiSelect *obj)
     spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
     gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spin),
       GTK_UPDATE_IF_VALID);
+    prev_value = 65;
+    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON (spin), TRUE);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj->window)->vbox),
       spin, FALSE, TRUE, 0);
 
